@@ -191,6 +191,8 @@ void DistributionList::setFields( const KABC::ContactGroup* contactGroup )
     Member m;
     m.displayName = data.name();
     m.email = data.email();
+
+    mDistrListMembers.append( m );
   }
 
   // Hopefully all resources are available during saving, so we can look up
@@ -199,7 +201,7 @@ void DistributionList::setFields( const KABC::ContactGroup* contactGroup )
   for ( uint index = 0; index < contactGroup->contactReferenceCount(); ++index ) {
     const KABC::ContactGroup::ContactReference& reference = contactGroup->contactReference( index );
 
-    const Akonadi::Item item( reference.uid() );
+    const Akonadi::Item item( reference.uid().toLongLong() );
     Akonadi::ItemFetchJob *job = new Akonadi::ItemFetchJob( item );
     job->fetchScope().fetchFullPayload();
     if ( !job->exec() )
@@ -217,6 +219,7 @@ void DistributionList::setFields( const KABC::ContactGroup* contactGroup )
       m.email = reference.preferredEmail();
       if ( m.email.isEmpty() )
         m.email = addressee.preferredEmail();
+
       mDistrListMembers.append( m );
     }
   }
