@@ -24,12 +24,13 @@
 #include "retrieveitemsjob.h"
 
 #include <QtCore/QDir>
-#include <QtCore/QFileSystemWatcher>
 #include <QtDBus/QDBusConnection>
 
 #include <akonadi/kmime/messageparts.h>
 #include <akonadi/changerecorder.h>
 #include <akonadi/itemfetchscope.h>
+#include <akonadi/itemfetchjob.h>
+#include <akonadi/itemmodifyjob.h>
 #include <akonadi/collectionfetchscope.h>
 #include <akonadi/cachepolicy.h>
 #include <akonadi/collectionfetchjob.h>
@@ -38,11 +39,11 @@
 #include <kmime/kmime_message.h>
 
 #include <kdebug.h>
+#include <kdirwatch.h>
 #include <kurl.h>
 #include <kfiledialog.h>
 #include <klocale.h>
-
-#include <KWindowSystem>
+#include <kwindowsystem.h>
 
 #include "libmaildir/maildir.h"
 
@@ -86,7 +87,7 @@ Collection MaildirResource::collectionForMaildir(const Maildir& md) const
 MaildirResource::MaildirResource( const QString &id )
     :ResourceBase( id ),
     mSettings( new MaildirSettings( componentData().config() ) ),
-    m_fsWatcher( new QFileSystemWatcher( this ) )
+    m_fsWatcher( new KDirWatch( this ) )
 {
   new MaildirSettingsAdaptor( mSettings );
   DBusConnectionPool::threadConnection().registerObject( QLatin1String( "/Settings" ),
@@ -110,7 +111,7 @@ MaildirResource::MaildirResource( const QString &id )
   scope.setAncestorRetrieval( ItemFetchScope::None );
   setItemSynchronizationFetchScope( scope );
 
-  connect( m_fsWatcher, SIGNAL(directoryChanged(QString)), SLOT(slotDirChanged(QString)) );
+  connect( m_fsWatcher, SIGNAL(dirty(QString)), SLOT(slotDirChanged(QString)) );
 
   synchronizeCollectionTree();
 }
@@ -168,6 +169,13 @@ void MaildirResource::configure( WId windowId )
   if ( windowId )
     KWindowSystem::setMainWindow( &dlg, windowId );
   if ( dlg.exec() ) {
+      // if we have no name, or the default one,
+      // better use the name of the top level collection
+      // that looks nicer
+      if ( name().isEmpty() || name() == identifier() ) {
+        Maildir md( mSettings->path() );
+        setName( md.name() );
+      }
     emit configurationDialogAccepted();
   } else {
     emit configurationDialogRejected();
@@ -196,8 +204,16 @@ void MaildirResource::itemAdded( const Akonadi::Item & item, const Akonadi::Coll
       return;
     }
     const KMime::Message::Ptr mail = item.payload<KMime::Message::Ptr>();
+
+    QString path = dir.path();
+    m_fsWatcher->removeDir( path + QLatin1Literal("/new") );
+    m_fsWatcher->removeDir( path + QLatin1Literal("/cur") );
+
     const QString rid = dir.addEntry( mail->encodedContent() );
 
+    m_fsWatcher->addDir( path + QLatin1Literal("/new") );
+    m_fsWatcher->addDir( path + QLatin1Literal("/cur") );
+
     Item i( item );
     i.setRemoteId( rid );
     changeCommitted( i );
@@ -209,15 +225,24 @@ void MaildirResource::itemChanged( const Akonadi::Item& item, const QSet<QByteAr
       cancelTask( i18n("Unusable configuration.") );
       return;
     }
+    
 
-    bool payloadChanged = false;
+    bool bodyChanged = false;
+    bool flagsChanged = false;
+    bool headChanged = false;
     Q_FOREACH( const QByteArray &part, parts )  {
-      if( part.startsWith("PLD:") ) {
-        payloadChanged = true;
-        break;
+      if( part.startsWith("PLD:RFC822") ) {
+        bodyChanged = true;
+      }
+      if( part.startsWith("PLD:HEAD") ) {
+        headChanged = true;
+      }
+      if ( part.contains( "FLAGS" ) ) {
+        flagsChanged = true;
       }
     }
-    if ( mSettings->readOnly() || !payloadChanged ) {
+
+    if ( mSettings->readOnly() || ( !bodyChanged && !flagsChanged && !headChanged ) ) {
       changeProcessed();
       return;
     }
@@ -228,14 +253,45 @@ void MaildirResource::itemChanged( const Akonadi::Item& item, const QSet<QByteAr
         cancelTask( errMsg );
         return;
     }
-    // we can only deal with mail
-    if ( !item.hasPayload<KMime::Message::Ptr>() ) {
-        cancelTask( i18n("Error: Unsupported type.") );
-        return;
+
+    Item newItem( item );
+
+    if ( flagsChanged || bodyChanged || headChanged ) { //something has changed that we can deal with
+      const QString path = dir.path();
+      m_fsWatcher->removeDir( path + QLatin1Literal("/new") );
+      m_fsWatcher->removeDir( path + QLatin1Literal("/cur") );
+
+      if ( flagsChanged ) { //flags changed, store in file name and get back the new filename (id)
+        const QString newKey = dir.changeEntryFlags( item.remoteId(), item.flags() );
+        newItem.setRemoteId( newKey );
+      }
+
+      if ( bodyChanged || headChanged ) { //head or body changed
+        // we can only deal with mail
+        if ( item.hasPayload<KMime::Message::Ptr>() ) {
+          const KMime::Message::Ptr mail = item.payload<KMime::Message::Ptr>();
+          QByteArray data = mail->encodedContent();
+          if ( headChanged && !bodyChanged ) { 
+            //only the head has changed, get the current version of the mail
+            //replace the head and store the new mail in the file
+            QByteArray currentData = dir.readEntry( newItem.remoteId() );
+            QByteArray newHead = mail->head();
+            mail->setContent( currentData );
+            mail->setHead( newHead );
+            mail->parse();
+            data = mail->encodedContent();
+          }
+          dir.writeEntry( newItem.remoteId(), data );
+        }
+      }
+
+      m_fsWatcher->addDir( path + QLatin1Literal("/new") );
+      m_fsWatcher->addDir( path + QLatin1Literal("/cur") );
+
+      changeCommitted( newItem );
+    } else {
+      emit changeProcessed();
     }
-    const KMime::Message::Ptr mail = item.payload<KMime::Message::Ptr>();
-    dir.writeEntry( item.remoteId(), mail->encodedContent() );
-    changeCommitted( item );
 }
 
 void MaildirResource::itemMoved( const Item &item, const Collection &source, const Collection &destination )
@@ -263,7 +319,20 @@ void MaildirResource::itemMoved( const Item &item, const Collection &source, con
     return;
   }
 
+  QString sourceDirPath = sourceDir.path();
+  QString destDirPath = destDir.path();
+  m_fsWatcher->removeDir( sourceDirPath + QLatin1Literal("/new") );
+  m_fsWatcher->removeDir( sourceDirPath + QLatin1Literal("/cur") );
+  m_fsWatcher->removeDir( destDirPath + QLatin1Literal("/new") );
+  m_fsWatcher->removeDir( destDirPath + QLatin1Literal("/cur") );
+
   const QString newRid = sourceDir.moveEntryTo( item.remoteId(), destDir );
+
+  m_fsWatcher->addDir( sourceDirPath + QLatin1Literal("/new") );
+  m_fsWatcher->addDir( sourceDirPath + QLatin1Literal("/cur") );
+  m_fsWatcher->addDir( destDirPath + QLatin1Literal("/new") );
+  m_fsWatcher->addDir( destDirPath + QLatin1Literal("/cur") );
+
   if ( newRid.isEmpty() ) {
     cancelTask( i18n( "Could not move message '%1'.", item.remoteId() ) );
     return;
@@ -285,17 +354,29 @@ void MaildirResource::itemRemoved(const Akonadi::Item & item)
     Maildir dir = maildirForCollection( item.parentCollection() );
     // !dir.isValid() means that our parent folder has been deleted already,
     // so we don't care at all as that one will be recursive anyway
+    QString dirPath = dir.path();
+    m_fsWatcher->removeDir( dirPath + QLatin1Literal("/new") );
+    m_fsWatcher->removeDir( dirPath + QLatin1Literal("/cur") );
     if ( dir.isValid() && !dir.removeEntry( item.remoteId() ) ) {
       emit error( i18n("Failed to delete message: %1", item.remoteId()) );
     }
+    m_fsWatcher->addDir( dirPath + QLatin1Literal("/new") );
+    m_fsWatcher->addDir( dirPath + QLatin1Literal("/cur") );
   }
+  qDebug() << "Item removed";
   changeProcessed();
 }
 
 Collection::List MaildirResource::listRecursive( const Collection &root, const Maildir &dir )
 {
-  if ( !dir.isRoot() && mSettings->monitorFilesystem() )
-    m_fsWatcher->addPath( dir.path() + QDir::separator() + QLatin1String( "new" ) );
+  if ( mSettings->monitorFilesystem() ) {
+    m_fsWatcher->addDir( dir.path() + QDir::separator() + QLatin1String( "new" ) );
+    m_fsWatcher->addDir( dir.path() + QDir::separator() + QLatin1String( "cur" ) );
+    m_fsWatcher->addDir( dir.subDirPath() );
+    if ( dir.isRoot() ) {
+      m_fsWatcher->addDir( dir.path() );
+    }
+  }
 
   Collection::List list;
   const QStringList mimeTypes = QStringList() << itemMimeType() << Collection::mimeType();
@@ -318,9 +399,6 @@ Collection::List MaildirResource::listRecursive( const Collection &root, const M
 
 void MaildirResource::retrieveCollections()
 {
-  if ( !m_fsWatcher->directories().isEmpty() )
-    m_fsWatcher->removePaths( m_fsWatcher->directories() );
-
   Maildir dir( mSettings->path(), mSettings->topLevelIsContainer() );
   QString errMsg;
   if ( !dir.isValid( errMsg ) ) {
@@ -508,7 +586,7 @@ void MaildirResource::collectionRemoved( const Akonadi::Collection &collection )
 void MaildirResource::ensureDirExists()
 {
   Maildir root( mSettings->path() );
-  if ( !root.isValid() ) {
+  if ( !root.isValid() && !mSettings->topLevelIsContainer() ) {
     if ( !root.create() )
       emit status( Broken, i18n( "Unable to create maildir '%1'.", mSettings->path() ) );
   }
@@ -525,7 +603,24 @@ bool MaildirResource::ensureSaneConfiguration()
 }
 
 void MaildirResource::slotDirChanged(const QString& dir)
-{
+{  
+  QFileInfo fileInfo(dir);
+  if (fileInfo.isFile()) {
+    slotFileChanged(dir);
+    return;
+  }
+  
+  if (dir == mSettings->path() ) {
+    synchronizeCollectionTree();
+   synchronizeCollection( Collection::root().id() );
+    return;
+  }
+
+  if ( dir.endsWith( QLatin1String( ".directory") ) ) {
+    synchronizeCollectionTree(); //might be too much, but this is not a common case anyway
+    return;
+  }
+  
   QDir d( dir );
   if ( !d.cdUp() )
     return;
@@ -539,12 +634,12 @@ void MaildirResource::slotDirChanged(const QString& dir)
     kDebug() << "unable to find collection for path" << dir;
     return;
   }
-
+ 
   CollectionFetchJob *job = new CollectionFetchJob( col, Akonadi::CollectionFetchJob::Base, this );
-  connect( job, SIGNAL(result(KJob*)), SLOT(fsWatchFetchResult(KJob*)) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(fsWatchDirFetchResult(KJob*)) );
 }
 
-void MaildirResource::fsWatchFetchResult(KJob* job)
+void MaildirResource::fsWatchDirFetchResult(KJob* job)
 {
   if ( job->error() ) {
     kDebug() << job->errorString();
@@ -553,7 +648,85 @@ void MaildirResource::fsWatchFetchResult(KJob* job)
   const Collection::List cols = qobject_cast<CollectionFetchJob*>( job )->collections();
   if ( cols.isEmpty() )
     return;
+  
   synchronizeCollection( cols.first().id() );
 }
 
+void MaildirResource::slotFileChanged( const QString& fileName )
+{
+  QFileInfo fileInfo( fileName );
+  QString key = fileInfo.fileName();
+  QString path = fileInfo.path();
+  if ( path.endsWith( QLatin1String("/new") ) ) {
+    path.remove( path.length() - 4, 4 );
+  } else if ( path.endsWith( QLatin1String("/cur") ) ) {
+    path.remove( path.length() - 4, 4 );
+  }
+  
+  const Maildir md( path );
+  if ( !md.isValid() )
+    return;
+      
+  const Collection col = collectionForMaildir( md );
+  if ( col.remoteId().isEmpty() ) {
+    kDebug() << "unable to find collection for path" << fileInfo.path();
+    return;
+  }
+
+  Item item;
+  item.setRemoteId( key );
+  item.setParentCollection( col );
+
+  ItemFetchJob *job = new ItemFetchJob( item, this );
+  job->setProperty("entry", key );
+  job->setProperty("dir", path );
+  connect( job, SIGNAL(result(KJob*)), SLOT(fsWatchFileFetchResult(KJob*)) );
+}
+
+void MaildirResource::fsWatchFileFetchResult( KJob* job )
+{
+  if ( job->error() ) {
+    kDebug() << job->errorString();
+    return;
+  }
+  Item::List items = qobject_cast<ItemFetchJob*>( job )->items();
+  if ( items.isEmpty() )
+    return;
+
+  QString fileName = job->property( "entry" ).toString();
+  QString path = job->property( "dir" ).toString();
+
+  const Maildir md( path );
+
+  QString entry = fileName;
+  Item item ( items.at(0) );
+  const qint64 entrySize = md.size( entry );
+  if ( entrySize >= 0 )
+    item.setSize( entrySize );
+
+  Item::Flags flags = md.readEntryFlags( entry );
+  Q_FOREACH( Item::Flag flag, flags ) {
+    item.setFlag(flag);      
+  }
+    
+  const QByteArray data = md.readEntry( entry );
+  KMime::Message *mail = new KMime::Message();
+  mail->setContent( KMime::CRLFtoLF( data ) );
+  mail->parse();
+
+  item.setPayload( KMime::Message::Ptr( mail ) );
+    
+  ItemModifyJob *mjob = new ItemModifyJob( item );
+  connect( mjob, SIGNAL(result(KJob*)), SLOT(fsWatchFileModifyResult(KJob*)) );
+}
+
+void MaildirResource::fsWatchFileModifyResult(KJob* job)
+{
+  if ( job->error() ) {
+    kDebug() << job->errorString();
+    return;
+  }
+}
+
+
 #include "maildirresource.moc"
