@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2009 Bertjan Broeksem <b.broeksema@kdemail.net>
+    Copyright (c) 2009 Bertjan Broeksem <broeksema@kde.org>
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -19,42 +19,44 @@
 
 #include "mboxresource.h"
 
+#include <QtCore/QtPlugin>
+
 #include <akonadi/attributefactory.h>
+#include <akonadi/agentfactory.h>
 #include <akonadi/changerecorder.h>
 #include <akonadi/collectionfetchjob.h>
 #include <akonadi/collectionmodifyjob.h>
+#include <akonadi/dbusconnectionpool.h>
 #include <akonadi/itemfetchscope.h>
+#include <kmbox/mbox.h>
 #include <kmime/kmime_message.h>
 #include <KWindowSystem>
-#include <QtDBus/QDBusConnection>
 
 #include "compactpage.h"
 #include "deleteditemsattribute.h"
 #include "lockmethodpage.h"
-#include "mbox.h"
 #include "settingsadaptor.h"
 #include "singlefileresourceconfigdialog.h"
 
 using namespace Akonadi;
-using namespace AkonadiFuture;
 
 static Entity::Id collectionId( const QString &remoteItemId )
 {
-  // [CollectionId]:[RemoteCollectionId]:[Offset]
+  // [CollectionId]::[RemoteCollectionId]::[Offset]
   Q_ASSERT( remoteItemId.split( "::" ).size() == 3 );
   return remoteItemId.split( "::" ).first().toLongLong();
 }
 
 static QString mboxFile(const QString &remoteItemId)
 {
-  // [CollectionId]:[RemoteCollectionId]:[Offset]
+  // [CollectionId]::[RemoteCollectionId]::[Offset]
   Q_ASSERT(remoteItemId.split( "::" ).size() == 3);
   return remoteItemId.split( "::" ).at(1);
 }
 
 static quint64 itemOffset( const QString &remoteItemId )
 {
-  // [CollectionId]:[RemoteCollectionId]:[Offset]
+  // [CollectionId]::[RemoteCollectionId]::[Offset]
   Q_ASSERT( remoteItemId.split( "::" ).size() == 3 );
   return remoteItemId.split( "::" ).last().toULongLong();
 }
@@ -63,9 +65,9 @@ MboxResource::MboxResource( const QString &id )
   : SingleFileResource<Settings>( id )
   , mMBox( 0 )
 {
-  new SettingsAdaptor( Settings::self() );
-  QDBusConnection::sessionBus().registerObject( QLatin1String( "/Settings" ),
-                              Settings::self(), QDBusConnection::ExportAdaptors );
+  new SettingsAdaptor( mSettings );
+  DBusConnectionPool::threadConnection().registerObject( QLatin1String( "/Settings" ),
+                                                         mSettings, QDBusConnection::ExportAdaptors );
 
   QStringList mimeTypes;
   mimeTypes << "message/rfc822";
@@ -79,37 +81,27 @@ MboxResource::~MboxResource()
 {
 }
 
-void MboxResource::configure( WId windowId )
+void MboxResource::customizeConfigDialog( SingleFileResourceConfigDialog<Settings>* dlg )
 {
-  QPointer<SingleFileResourceConfigDialog<Settings> > dlg
-    = new SingleFileResourceConfigDialog<Settings>( windowId );
-
-  dlg->addPage( "Compact frequency", new CompactPage( Settings::self()->path() ) );
-  dlg->addPage( "Lock method", new LockMethodPage() );
+  dlg->addPage( i18n("Compact frequency"), new CompactPage( mSettings->path() ) );
+  dlg->addPage( i18n("Lock method"), new LockMethodPage() );
   dlg->setCaption( i18n( "Select MBox file" ) );
-  if ( dlg->exec() == QDialog::Accepted ) {
-    reloadFile();
-
-    emit configurationDialogAccepted();
-  } else {
-    emit configurationDialogRejected();
-  }
-
-  delete dlg;
 }
 
 void MboxResource::retrieveItems( const Akonadi::Collection &col )
 {
   Q_UNUSED( col );
-  if ( !mMBox )
+  if ( !mMBox ) {
+    cancelTask();
     return;
+  }
 
-  QList<MsgInfo> entryList;
+  KMBox::MBoxEntry::List entryList;
   if ( col.hasAttribute<DeletedItemsAttribute>() ) {
     DeletedItemsAttribute *attr = col.attribute<DeletedItemsAttribute>();
-    entryList = mMBox->entryList( attr->deletedItemOffsets() );
+    entryList = mMBox->entries( attr->deletedItemEntries() );
   } else { // No deleted items (yet)
-    entryList = mMBox->entryList();
+    entryList = mMBox->entries();
   }
 
   mMBox->lock(); // Lock the file so that it doesn't get locked for every
@@ -119,19 +111,19 @@ void MboxResource::retrieveItems( const Akonadi::Collection &col )
   QString colId = QString::number( col.id() );
   QString colRid = col.remoteId();
   double count = 1;
-  foreach ( const MsgInfo &entry, entryList ) {
+  foreach ( const KMBox::MBoxEntry &entry, entryList ) {
     // TODO: Use cache policy to see what actually has to been set as payload.
     //       Currently most views need a minimal amount of information so the
     //       Items get Envelopes as payload.
     KMime::Message *mail = new KMime::Message();
-    mail->setHead( KMime::CRLFtoLF( mMBox->readEntryHeaders( entry.first ) ) );
+    mail->setHead( KMime::CRLFtoLF( mMBox->readMessageHeaders( entry ) ) );
     mail->parse();
 
     Item item;
-    item.setRemoteId( colId + "::" + colRid + "::" + QString::number( entry.first ) );
+    item.setRemoteId( colId + "::" + colRid + "::" + QString::number( entry.messageOffset() ) );
     item.setMimeType( "message/rfc822" );
-    item.setSize( entry.second );
-    item.setPayload( MessagePtr( mail ) );
+    item.setSize( entry.messageSize() );
+    item.setPayload( KMime::Message::Ptr( mail ) );
 
     emit percent(count++ / entryList.size());
     items << item;
@@ -152,24 +144,24 @@ bool MboxResource::retrieveItem( const Akonadi::Item &item, const QSet<QByteArra
   }
 
   QString rid = item.remoteId();
-  quint64 offset = itemOffset( rid );
-  KMime::Message *mail = mMBox->readEntry( offset );
+  const quint64 offset = itemOffset( rid );
+  KMime::Message *mail = mMBox->readMessage( KMBox::MBoxEntry( offset ) );
   if ( !mail ) {
     emit error( i18n( "Failed to read message with uid '%1'.", rid ) );
     return false;
   }
 
   Item i( item );
-  i.setPayload( MessagePtr( mail ) );
+  i.setPayload( KMime::Message::Ptr( mail ) );
   itemRetrieved( i );
   return true;
 }
 
 void MboxResource::aboutToQuit()
 {
-  if ( !Settings::self()->readOnly() )
+  if ( !mSettings->readOnly() )
     writeFile();
-  Settings::self()->writeConfig();
+  mSettings->writeConfig();
 }
 
 void MboxResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
@@ -180,20 +172,20 @@ void MboxResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collecti
   }
 
   // we can only deal with mail
-  if ( !item.hasPayload<MessagePtr>() ) {
+  if ( !item.hasPayload<KMime::Message::Ptr>() ) {
     cancelTask( i18n( "Only email messages can be added to the MBox resource." ) );
     return;
   }
 
-  qint64 offset = mMBox->appendEntry( item.payload<MessagePtr>() );
-  if ( offset == -1 ) {
+  const KMBox::MBoxEntry entry = mMBox->appendMessage( item.payload<KMime::Message::Ptr>() );
+  if ( !entry.isValid() ) {
     cancelTask( i18n( "Mail message not added to the MBox." ) );
     return;
   }
 
   scheduleWrite();
   const QString rid = QString::number( collection.id() ) + "::"
-                      + collection.remoteId() + "::" + QString::number( offset );
+                      + collection.remoteId() + "::" + QString::number( entry.messageOffset() );
 
   Item i( item );
   i.setRemoteId( rid );
@@ -240,10 +232,10 @@ void MboxResource::itemRemoved( const Akonadi::Item &item )
   DeletedItemsAttribute *attr
     = mboxCollection.attribute<DeletedItemsAttribute>( Akonadi::Entity::AddIfMissing );
 
-  if ( Settings::self()->compactFrequency() == Settings::per_x_messages
-       && Settings::self()->messageCount() == static_cast<uint>( attr->offsetCount() + 1 ) ) {
+  if ( mSettings->compactFrequency() == Settings::per_x_messages
+       && mSettings->messageCount() == static_cast<uint>( attr->offsetCount() + 1 ) ) {
     kDebug() << "Compacting mbox file";
-    mMBox->purge( attr->deletedItemOffsets() << itemOffset( item.remoteId() ) );
+    mMBox->purge( attr->deletedItemEntries() << KMBox::MBoxEntry( itemOffset( item.remoteId() ) ) );
     scheduleWrite();
     mboxCollection.removeAttribute<DeletedItemsAttribute>();
   } else {
@@ -271,20 +263,18 @@ void MboxResource::handleHashChange()
 bool MboxResource::readFromFile( const QString &fileName )
 {
   delete mMBox;
-  mMBox = 0;
+  mMBox = new KMBox::MBox();
 
-  mMBox = new MBox();
-
-  switch ( Settings::self()->lockfileMethod() ) {
+  switch ( mSettings->lockfileMethod() ) {
     case Settings::procmail:
-      mMBox->setLockType( MBox::ProcmailLockfile );
-      mMBox->setLockFile( Settings::self()->lockfile() );
+      mMBox->setLockType( KMBox::MBox::ProcmailLockfile );
+      mMBox->setLockFile( mSettings->lockfile() );
       break;
     case Settings::mutt_dotlock:
-      mMBox->setLockType( MBox::MuttDotlock );
+      mMBox->setLockType( KMBox::MBox::MuttDotlock );
       break;
     case Settings::mutt_dotlock_privileged:
-      mMBox->setLockType( MBox::MuttDotlockPrivileged );
+      mMBox->setLockType( KMBox::MBox::MuttDotlockPrivileged );
       break;
   }
 
@@ -365,6 +355,6 @@ void MboxResource::onCollectionModify( KJob *job )
   itemAdded( item, c );
 }
 
-AKONADI_RESOURCE_MAIN( MboxResource )
+AKONADI_AGENT_FACTORY( MboxResource, akonadi_mbox_resource )
 
 #include "mboxresource.moc"
