@@ -393,6 +393,10 @@ class MaildirContext
       mMaildir = Maildir( newPath, mMaildir.isRoot() );
     }
 
+    const Maildir & maildir() const {
+        return mMaildir;
+    }
+
   private:
     Maildir mMaildir;
 
@@ -483,8 +487,8 @@ class MixedMaildirStore::Private : public FileStore::Job::Visitor
     void fillMaildirTreeDetails( const Maildir &md, const Collection &collection, Collection::List &collections, bool recurse );
     void listCollection( FileStore::Job *job, MBoxPtr &mbox, const Collection &collection, Item::List &items );
     void listCollection( FileStore::Job *job, MaildirPtr &md, const Collection &collection, Item::List &items );
-    bool fillItem( MBoxPtr &mbox, bool includeBody, Item &item ) const;
-    bool fillItem( const MaildirPtr &md, bool includeBody, Item &item ) const;
+    bool fillItem( MBoxPtr &mbox, bool includeHeaders, bool includeBody, Item &item ) const;
+    bool fillItem( const MaildirPtr &md, bool includeHeaders, bool includeBody, Item &item ) const;
     void updateContextHashes( const QString &oldPath, const QString &newPath );
 
   public: // visitor interface implementation
@@ -791,6 +795,10 @@ void MixedMaildirStore::Private::listCollection( FileStore::Job *job, MaildirPtr
         }
       }
     }
+    Akonadi::Item::Flags flags = md->maildir().readEntryFlags( entry );
+    Q_FOREACH( Akonadi::Item::Flag flag, flags ) {
+      item.setFlag(flag);
+    }
 
     items << item;
   }
@@ -810,7 +818,7 @@ void MixedMaildirStore::Private::listCollection( FileStore::Job *job, MaildirPtr
   }
 }
 
-bool MixedMaildirStore::Private::fillItem( MBoxPtr &mbox, bool includeBody, Item &item ) const
+bool MixedMaildirStore::Private::fillItem( MBoxPtr &mbox, bool includeHeaders, bool includeBody, Item &item ) const
 {
 //  kDebug( KDE_DEFAULT_DEBUG_AREA ) << "Filling item" << item.remoteId() << "from MBox: includeBody=" << includeBody;
   bool ok = false;
@@ -818,44 +826,57 @@ bool MixedMaildirStore::Private::fillItem( MBoxPtr &mbox, bool includeBody, Item
   if ( !ok || !mbox->isValidOffset( offset ) ) {
     return false;
   }
+  
+  // TODO: size and modification timestamp?
 
-  KMime::Message::Ptr messagePtr( new KMime::Message() );
-  if ( includeBody ) {
-    const QByteArray data = mbox->readRawEntry( offset );
-    messagePtr->setContent( KMime::CRLFtoLF( data ) );
-  } else {
-    const QByteArray data = mbox->readEntryHeaders( offset );
-    messagePtr->setHead( KMime::CRLFtoLF( data ) );
+  if ( includeHeaders || includeBody ) {
+    KMime::Message::Ptr messagePtr( new KMime::Message() );
+    if ( includeBody ) {
+      const QByteArray data = mbox->readRawEntry( offset );
+      messagePtr->setContent( KMime::CRLFtoLF( data ) );
+    } else {
+      const QByteArray data = mbox->readEntryHeaders( offset );
+      messagePtr->setHead( KMime::CRLFtoLF( data ) );
+    }
+    messagePtr->parse();
+
+    item.setPayload<KMime::Message::Ptr>( messagePtr );
   }
-  messagePtr->parse();
-
-  item.setPayload<KMime::Message::Ptr>( messagePtr );
   return true;
 }
 
-bool MixedMaildirStore::Private::fillItem( const MaildirPtr &md, bool includeBody, Item &item ) const
+bool MixedMaildirStore::Private::fillItem( const MaildirPtr &md, bool includeHeaders, bool includeBody, Item &item ) const
 {
 /*  kDebug( KDE_DEFAULT_DEBUG_AREA ) << "Filling item" << item.remoteId() << "from Maildir: includeBody=" << includeBody;*/
-  KMime::Message::Ptr messagePtr( new KMime::Message() );
 
-  if ( includeBody ) {
-    const QByteArray data = md->readEntry( item.remoteId() );
-    if ( data.isEmpty() ) {
-      return false;
+  const qint64 entrySize = md->maildir().size( item.remoteId() );
+  if ( entrySize < 0 )
+    return false;
+  
+  item.setSize( entrySize );
+  item.setModificationTime( md->maildir().lastModified( item.remoteId() ) );
+
+  if ( includeHeaders || includeBody ) {
+    KMime::Message::Ptr messagePtr( new KMime::Message() );
+    if ( includeBody ) {
+      const QByteArray data = md->readEntry( item.remoteId() );
+      if ( data.isEmpty() ) {
+        return false;
+      }
+
+      messagePtr->setContent( KMime::CRLFtoLF( data ) );
+    } else {
+      const QByteArray data = md->readEntryHeaders( item.remoteId() );
+      if ( data.isEmpty() ) {
+        return false;
+      }
+
+      messagePtr->setHead( KMime::CRLFtoLF( data ) );
     }
+    messagePtr->parse();
 
-    messagePtr->setContent( KMime::CRLFtoLF( data ) );
-  } else {
-    const QByteArray data = md->readEntryHeaders( item.remoteId() );
-    if ( data.isEmpty() ) {
-      return false;
-    }
-
-    messagePtr->setHead( KMime::CRLFtoLF( data ) );
+    item.setPayload<KMime::Message::Ptr>( messagePtr );
   }
-  messagePtr->parse();
-
-  item.setPayload<KMime::Message::Ptr>( messagePtr );
   return true;
 }
 
@@ -1587,6 +1608,8 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemFetchJob *job )
   ItemFetchScope scope = job->fetchScope();
   const bool includeBody = scope.fullPayload() ||
                            scope.payloadParts().contains( MessagePart::Body );
+  const bool includeHeaders = scope.payloadParts().contains( MessagePart::Header ) ||
+                              scope.payloadParts().contains( MessagePart::Envelope );
 
   const bool fetchSingleItem = job->collection().remoteId().isEmpty();
   const Collection collection = fetchSingleItem ? job->item().parentCollection() : job->collection();
@@ -1630,7 +1653,7 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemFetchJob *job )
     Item::List::iterator it    = items.begin();
     Item::List::iterator endIt = items.end();
     for ( ; it != endIt; ++it ) {
-      if ( !fillItem( findIt.value(), includeBody, *it ) ) {
+      if ( !fillItem( findIt.value(), includeHeaders, includeBody, *it ) ) {
         const QString errorText =
           i18nc( "@info:status", "Error while reading mails from folder %1", collection.name() );
         q->notifyError( FileStore::Job::InvalidJobContext, errorText ); // TODO should be a different error code
@@ -1669,7 +1692,7 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemFetchJob *job )
     Item::List::iterator it    = items.begin();
     Item::List::iterator endIt = items.end();
     for ( ; it != endIt; ++it ) {
-      if ( !fillItem( mdPtr, includeBody, *it ) ) {
+      if ( !fillItem( mdPtr, includeHeaders, includeBody, *it ) ) {
         const QString errorText =
           i18nc( "@info:status", "Error while reading mails from folder %1", collection.name() );
         q->notifyError( FileStore::Job::InvalidJobContext, errorText ); // TODO should be a different error code
@@ -1688,11 +1711,23 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemFetchJob *job )
 
 bool MixedMaildirStore::Private::visit( FileStore::ItemModifyJob *job )
 {
-  // if we can ignore payload, we have nothing to do
-  if ( job->ignorePayload() ) {
-    return true;
+  const QSet<QByteArray> parts = job->parts();
+  bool payloadChanged = false;
+  bool flagsChanged = false;
+  Q_FOREACH( const QByteArray &part, parts )  {
+    if( part.startsWith( "PLD:" ) ) {
+      payloadChanged = true;
+    }
+    if ( part.contains( "FLAGS" ) ) {
+      flagsChanged = true;
+    }
   }
 
+  const bool nothingChanged = ( !payloadChanged && !flagsChanged );
+  const bool payloadChangedButIgnored = payloadChanged && job->ignorePayload();
+  const bool ignoreModifyIfValid = nothingChanged ||
+                                   ( payloadChangedButIgnored && !flagsChanged );
+  
   Item item = job->item();
   const Collection collection = item.parentCollection();
   QString path;
@@ -1735,6 +1770,24 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemModifyJob *job )
       return false;
     }
 
+    // if we can ignore payload, or we have nothing else to change, then we are finished
+    if ( ignoreModifyIfValid ) {
+      kDebug( KDE_DEFAULT_DEBUG_AREA ) << "ItemModifyJob for item" << item.remoteId()
+          << "in collection" << collection.remoteId()
+          << "skipped: nothing of interest changed (" << nothingChanged
+          << ") or only payload changed but should be ignored ("
+          << ( payloadChanged && !flagsChanged && job->ignorePayload() )
+          << "). Modified parts:" << parts;
+      q->notifyItemsProcessed( Item::List() << job->item() );
+      return true;
+    }
+
+    // mbox can only change payload, ignore any other change
+    if ( !payloadChanged ) {
+      q->notifyItemsProcessed( Item::List() << item );
+      return true;
+    }
+ 
     // make sure to read the index (if available) before modifying the data, which would
     // make the index invalid
     mbox->readIndexData();
@@ -1780,6 +1833,18 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemModifyJob *job )
       return false;
     }
 
+    // if we can ignore payload, or we have nothing else to change, then we are finished
+    if ( ignoreModifyIfValid ) {
+      kDebug( KDE_DEFAULT_DEBUG_AREA ) << "ItemModifyJob for item" << item.remoteId()
+          << "in collection" << collection.remoteId()
+          << "skipped: nothing of interest changed (" << nothingChanged
+          << ") or only payload changed but should be ignored ("
+          << ( payloadChanged && !flagsChanged && job->ignorePayload() )
+          << "). Modified parts:" << parts;
+      q->notifyItemsProcessed( Item::List() << job->item() );
+      return true;
+    }
+
     // make sure to read the index (if available) before modifying the data, which would
     // make the index invalid
     mdPtr->readIndexData();
@@ -1791,7 +1856,16 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemModifyJob *job )
       job->setProperty( "onDiskIndexInvalidated", var );
     }
 
-    mdPtr->writeEntry( item.remoteId(), item.payload<KMime::Message::Ptr>()->encodedContent() );
+    QString newKey = item.remoteId();
+    if ( flagsChanged ) {
+      Maildir md( mdPtr->maildir() );
+      newKey = md.changeEntryFlags( item.remoteId(), item.flags() );
+      item.setRemoteId( newKey );
+    }
+
+    if ( payloadChanged ) {
+      mdPtr->writeEntry( newKey, item.payload<KMime::Message::Ptr>()->encodedContent() );
+    }
   }
 
   q->notifyItemsProcessed( Item::List() << item );
@@ -1874,7 +1948,7 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemMoveJob *job )
 
     if ( !item.hasPayload<KMime::Message::Ptr>() ||
          !item.loadedPayloadParts().contains( MessagePart::Body ) ) {
-      if ( !fillItem( mbox, true, item ) ) {
+      if ( !fillItem( mbox, true, true, item ) ) {
         errorText = i18nc( "@info:status", "Cannot move email from folder %1",
                             sourceCollection.name() );
         kError() << errorText << "FolderType=" << sourceFolderType;
@@ -2003,7 +2077,7 @@ bool MixedMaildirStore::Private::visit( FileStore::ItemMoveJob *job )
 /*      kDebug( KDE_DEFAULT_DEBUG_AREA ) << "target is MBox";*/
       if ( !item.hasPayload<KMime::Message::Ptr>() ||
            !item.loadedPayloadParts().contains( MessagePart::Body ) ) {
-        if ( !fillItem( sourceMdPtr, true, item ) ) {
+        if ( !fillItem( sourceMdPtr, true, true, item ) ) {
           errorText = i18nc( "@info:status", "Cannot move email from folder %1",
                               sourceCollection.name() );
           kError() << errorText << "FolderType=" << sourceFolderType;
@@ -2183,6 +2257,7 @@ void MixedMaildirStore::setTopLevelCollection( const Collection &collection )
   cachePolicy.setInheritFromParent( false );
   cachePolicy.setLocalParts( QStringList() << MessagePart::Envelope );
   cachePolicy.setSyncOnDemand( true );
+  cachePolicy.setCacheTimeout( 1 );
 
   Collection modifiedCollection = collection;
   modifiedCollection.setContentMimeTypes( contentMimeTypes );
