@@ -24,7 +24,6 @@
 #include <qtest_kde.h>
 
 using namespace Akonadi;
-using namespace AkonadiFuture;
 
 class XmlDocumentTest : public QObject
 {
@@ -53,7 +52,7 @@ class XmlDocumentTest : public QObject
       item = items.first();
       QVERIFY( item.hasPayload() );
       QCOMPARE( item.flags().count(), 1 );
-      QVERIFY( item.hasFlag( "\\Seen" ) );
+      QVERIFY( item.hasFlag( "\\SEEN" ) );
     }
 };
 
